//+------------------------------------------------------------------+
//|                                           CalculateProfit_EA.mq5 |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

//--- name of the horizontal line that shows the entry price
string entry_line="entry";
//--- name of the horizontal line that shows the exit price
string exit_line="exit";
//--- arrow name
string arrow_line="direct arrow";
//--- text label name
string text_field="profit info";
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- draw the entry and exit lines on a chart
   CheckLevelLines();
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- delete created objects from chart
   ObjectDelete(0,entry_line);
   ObjectDelete(0,exit_line);
   ObjectDelete(0,arrow_line);
   ObjectDelete(0,text_field);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   CheckLevelLines();
//--- show the difference between the levels in terms of money on the chart
   ShowProfit();
  }
//+------------------------------------------------------------------+
//| Calculate potential profit/loss for buying 1 lot                 |
//+------------------------------------------------------------------+
double CalculateProfitOneLot(double entry_price,double exit_price)
  {
//--- write the profit value to this variable
   double profit=0;
   if(!OrderCalcProfit(ORDER_TYPE_BUY,Symbol(),1.0,entry_price,exit_price,profit))
     {
      Print(__FUNCTION__,"  Failed to calculate OrderCalcProfit(). Error ",GetLastError());
     }
//---
   return(profit);
  }
//+------------------------------------------------------------------+
//| Capture chart events                                             |
//+------------------------------------------------------------------+
void  OnChartEvent(const int id,         // event ID
                   const long& lparam,   // long parameter of event
                   const double& dparam, // double parameter of event
                   const string& sparam  // string parameter of event
                   )
  {
//--- show the difference between the levels in terms of money on the chart
   ShowProfit();
  }
//+------------------------------------------------------------------+
//|  calculate and display profit on the chart                       |
//+------------------------------------------------------------------+
void ShowProfit()
  {
//--- Entry level
   static double entry=0;
//--- Exit level
   static double exit=0;

//--- get the current values of the levels
   double curr_entry=NormalizeDouble(ObjectGetDouble(0,entry_line,OBJPROP_PRICE),_Digits);
   double curr_exit=NormalizeDouble(ObjectGetDouble(0,exit_line,OBJPROP_PRICE),_Digits);

//--- compare current and memorized values
   if(entry!=curr_entry || exit!=curr_exit)
     {
      //--- memorize new values
      entry=curr_entry;
      exit=curr_exit;
      //--- calculate the difference between the levels in terms of money
      double profit=CalculateProfitOneLot(entry,exit);
      //--- set color for the exit level
      SetColorOnProfit(profit);
      //--- show the arrow
      SetArrowedLine(entry,exit,profit);
      //--- show the value near to the center of the arrow using text
      SetTextInfo(entry,exit,profit);
     }
  }
//+------------------------------------------------------------------+
//| set color of the line depending on the sign - profit or loss     |
//+------------------------------------------------------------------+
void SetColorOnProfit(double profit)
  {
   if(profit>0)
     {
      //--- set the green color
      ObjectSetInteger(0,exit_line,OBJPROP_COLOR,Green);
      ChartRedraw();
     }
   else
     {
      //---- set the red color
      ObjectSetInteger(0,exit_line,OBJPROP_COLOR,Red);
      ChartRedraw();
     }
  }
//+------------------------------------------------------------------+
//| Draw arrow of required direction and color on zero bar           |
//+------------------------------------------------------------------+
void SetArrowedLine(double entry,double exit,double profit)
  {
//--- if there is no arrow
   if(ObjectFind(0,arrow_line)<0)
     {
      if(!ObjectCreate(0,arrow_line,OBJ_ARROWED_LINE,0,TimeCurrent(),entry,TimeCurrent(),exit))
         Print("Failed to create arrowed line. Error ",GetLastError());
     }
//--- set time coordinates for the arrow
   ObjectSetInteger(0,arrow_line,OBJPROP_TIME,0,TimeCurrent());
   ObjectSetInteger(0,arrow_line,OBJPROP_TIME,1,TimeCurrent());
//--- set price coordinates for the arrow
   ObjectSetDouble(0,arrow_line,OBJPROP_PRICE,0,entry);
   ObjectSetDouble(0,arrow_line,OBJPROP_PRICE,1,exit);
//--- set color for the arrow
   if(profit>0)
      ObjectSetInteger(0,arrow_line,OBJPROP_COLOR,Green);
   else
      ObjectSetInteger(0,arrow_line,OBJPROP_COLOR,Red);
//--- renew objects on the chart
   ChartRedraw();
  }
//+------------------------------------------------------------------+
//|  Write the difference in terms of money                          |
//+------------------------------------------------------------------+
void SetTextInfo(double entry,double exit,double profit)
  {
//--- if there is no arrow
   if(ObjectFind(0,text_field)<0)
     {
      if(!ObjectCreate(0,text_field,OBJ_TEXT,0,TimeCurrent()-PeriodSeconds(),(entry+exit)/2))
         Print("Failed to create text label. Error ",GetLastError());
     }
   else
     {
      //--- set time coordinates
      ObjectSetInteger(0,text_field,OBJPROP_TIME,0,TimeCurrent()-PeriodSeconds());
      ObjectSetDouble(0,text_field,OBJPROP_PRICE,0,(entry+exit)/2);
     }
//--- set color of the label
   if(profit>0)
      ObjectSetInteger(0,text_field,OBJPROP_COLOR,Green);
   else
      ObjectSetInteger(0,text_field,OBJPROP_COLOR,Red);
//--- set binding type
   ObjectSetInteger(0,text_field,OBJPROP_ANCHOR,ANCHOR_RIGHT);
//--- prepare text
   string info=StringFormat("%s: Volume=%.1f lots, Delta=%G %s",Symbol(),1.0,profit,AccountInfoString(ACCOUNT_CURRENCY));
//--- insert the text into the label
   ObjectSetString(0,text_field,OBJPROP_TEXT,info);
//--- renew objects on the chart
   ChartRedraw();
  }
//+------------------------------------------------------------------+
//| Create the entry and exit lines if they are not present          |
//+------------------------------------------------------------------+
void CheckLevelLines()
  {
//--- maximum price at the chart
   double maxPrice=0;
//--- minimum price at the chart
   double minPrice=0;
//--- value of price that will be taken for drawing the entry level
   double entry_level;
//--- value of price that will be taken for drawing the exit level
   double profit_level;

//--- check if there are levels
   bool entry_line_exist=(ObjectFind(0,entry_line)>=0);
   bool exit_line_exist=(ObjectFind(0,exit_line)>=0);

//--- if any of levels is missing, create it
   if(!entry_line_exist || !exit_line_exist)
     {
      //--- calculate vertical borders of the chart
      maxPrice=ChartGetDouble(0,CHART_PRICE_MAX);
      minPrice=ChartGetDouble(0,CHART_PRICE_MIN);
      //--- let the entry price be between the maximum and minimum prices
      entry_level=NormalizeDouble((maxPrice+minPrice)/2,_Digits);
      //--- let the exit price be less by one fourth 
      profit_level=NormalizeDouble(entry_level-(entry_level-minPrice)/2,_Digits);

      //--- create the corresponding objects
      if(!entry_line_exist)
        {
         ResetLastError();
         if(!ObjectCreate(0,entry_line,OBJ_HLINE,0,0,entry_level))
            Print("Failed to create the horizontal line named ",entry_line,". Error ",GetLastError());
         else
           {
            //--- give the possibility to select the object with the mouse
            ObjectSetInteger(0,entry_line,OBJPROP_SELECTABLE,true);
            //--- make the line selected to be able to move it right away
            ObjectSetInteger(0,entry_line,OBJPROP_SELECTED,true);
            //--- set color for the entry level
            ObjectSetInteger(0,entry_line,OBJPROP_COLOR,Brown);
           }
        }
      if(!exit_line_exist)
        {
         ResetLastError();
         if(!ObjectCreate(0,exit_line,OBJ_HLINE,0,0,profit_level))
            Print("Failed to create the horizontal line named ",exit_line,". Error ",GetLastError());
         else
           {
            //--- give the possibility to select the object with the mouse
            ObjectSetInteger(0,exit_line,OBJPROP_SELECTABLE,true);
            //--- make the line selected to be able to move it right away
            ObjectSetInteger(0,exit_line,OBJPROP_SELECTED,true);
            //--- set color for the entry level
            ObjectSetInteger(0,exit_line,OBJPROP_COLOR,Red);
           }
        }
     }
//---
  }
//+------------------------------------------------------------------+
